@@ -1,13 +1,16 @@
-import logging
-import os.path
-import random
-import asyncio
 import re
+import sys
 import time
+import random
+import os.path
+import logging
 import aiohttp
+import asyncio
 from w3lib.url import canonicalize_url
 from espider.request import Request
-from espider.utils import PriorityQueue, headers_to_dict, cookies_to_dict, get_md5
+from espider.utils import (
+    PriorityQueue, headers_to_dict, cookies_to_dict, get_md5, human_time
+)
 from espider.response import Response
 from inspect import isgenerator
 from pprint import pformat
@@ -48,7 +51,8 @@ class Spider(object):
         self.name = name or self.__class__.__name__
 
         self._msg_total_requests = 0
-        self._msg_runtime = 0
+        self._msg_runtime = 0.0
+        self._msg_runtime_fmt = ''
         self._msg_items = 0
         self._msg_item_speed = 0
         self._msg_item_dropped = 0
@@ -102,9 +106,26 @@ class Spider(object):
         self._start_time = time.time()
 
     def start(self):
-        self.prepare()
-        self.SPIDER_LOOP.run_until_complete(self.__run())
-        self.close()
+        try:
+            self.__init_logger()
+            self.prepare()
+            self.SPIDER_LOOP.run_until_complete(self.__run())
+            self.close()
+        except KeyboardInterrupt:
+            self.logger.warning('KeyboardInterrupt')
+            self.__close_msg()
+            try:
+                sys.exit(0)
+            except SystemExit:
+                os._exit(0)
+        except Exception as e:
+            self.__pretty_error(e)
+            if hasattr(self.REQUEST_SESSION, 'close'):
+                if hasattr(self.REQUEST_SESSION, 'closed') and not self.REQUEST_SESSION.closed:
+                    self.SPIDER_LOOP.run_until_complete(self.REQUEST_SESSION.close())
+            self.__close_msg()
+        else:
+            self.__close_msg()
 
     @property
     def msg(self):
@@ -167,17 +188,12 @@ class Spider(object):
         """
         主函数
         """
-        try:
-            self.__assert_params()
-            self.__init_logger()
-        except Exception as e:
-            self.logger.exception(e)
-        else:
-            consumer = asyncio.ensure_future(self.__downloader())
-            await self.__init_queue()
-            await self.REQUEST_QUEUE.join()
-            await self.REQUEST_SESSION.close()
-            consumer.cancel()
+        self.__assert_params()
+        consumer = asyncio.ensure_future(self.__downloader())
+        await self.__init_queue()
+        await self.REQUEST_QUEUE.join()
+        consumer.cancel()
+        await self.REQUEST_SESSION.close()
 
     def request(self, url=None, method=None, data=None, json=None, headers=None, cookies=None, callback=None,
                 cb_args=None, cb_kwargs=None, priority=None, allow_redirects=True, **kwargs):
@@ -208,40 +224,39 @@ class Spider(object):
         """
         异步请求
         """
+        # 处理请求
+        req = await self.__process_request(req)
+        if req is None: return
+
+        if self.REQUEST_DELAY: await asyncio.sleep(self.REQUEST_DELAY)
+
+        msg = self.__collect_msg(priority, req)
+        self.logger.info(msg)
+        # FIXME aiohttp.client_exceptions.ServerDisconnectedError
+        async with self.REQUEST_SESSION.request(**req.__dict__) as _resp:
+            data = await _resp.read()
+            resp = Response(_resp)
+            resp.text = data
+            resp.request = req
+            if self.REQUEST_WARNING and resp.status_code != 200:
+                detail = pformat(req.__dict__).replace('\n', '\n\t\t')
+                detail = re.sub('^\{', '{\n\t\t ', detail)
+                detail = re.sub('\}$', '\n\t}', detail)
+                msg = msg + ' {}\n\t{}\n'.format(resp.status_code, detail)
+                self.logger.warning(msg)
+
+        # 更新爬虫信息
+        self.__update_msg(priority)
+
+        # 处理响应
+        resp = await self.__process_response(resp)
+        if resp is None: return
+
+        # 处理回调函数
         try:
-            # 处理请求
-            req = await self.__process_request(req)
-            if req is None: return
-
-            if self.REQUEST_DELAY: await asyncio.sleep(self.REQUEST_DELAY)
-
-            msg = self.__collect_msg(priority, req)
-            self.logger.info(msg)
-            # FIXME aiohttp.client_exceptions.ServerDisconnectedError
-            async with self.REQUEST_SESSION.request(**req.__dict__) as _resp:
-                data = await _resp.read()
-                resp = Response(_resp)
-                resp.text = data
-                resp.request = req
-                if self.REQUEST_WARNING and resp.status_code != 200:
-                    detail = pformat(req.__dict__).replace('\n', '\n\t\t')
-                    detail = re.sub('^\{', '{\n\t\t ', detail)
-                    detail = re.sub('\}$', '\n\t}', detail)
-                    msg = msg + '{}\n\t{}\n'.format(resp.status_code, detail)
-                    self.logger.warning(msg)
-
-            # 更新爬虫信息
-            self.__update_msg(priority)
-
-            # 处理响应
-            resp = await self.__process_response(resp)
-            if resp is None: return
-
-        except Exception as e:
-            self.logger.exception(msg=f"Async Request Error: {e}")
-        else:
-            # 处理回调函数
             await self.__process_callback(priority, callback, resp, *args, **kwargs)
+        except Exception as e:
+            self.__pretty_error(e)
 
     async def __process_request(self, req):
 
@@ -264,8 +279,7 @@ class Spider(object):
             return
         return resp
 
-    @staticmethod
-    async def __process_middleware(req, middlewares):
+    async def __process_middleware(self, req, middlewares):
         if not middlewares: return req
         for mid in middlewares:
             req = mid(req)
@@ -277,22 +291,83 @@ class Spider(object):
         """
         处理回调函数
         """
-        try:
-            result = callback(resp, *args, **kwargs)
-            if isinstance(result, Coroutine): result = await result
-        except Exception as e:
-            self.logger.exception(msg='Process Callback({}) Error: {}'.format(callback.__name__, e))
+        result = callback(resp, *args, **kwargs)
+        if isinstance(result, Coroutine): result = await result
+        if not result: return
+        if isgenerator(result):
+            for r in result:
+                if not r: continue
+                if isinstance(r, tuple) and isinstance(r[1], Request):
+                    self.REQUEST_QUEUE.put_nowait(r)
+                else:
+                    await self.__process_item(priority, r)
         else:
-            if not result: return
-            if isgenerator(result):
-                for r in result:
-                    if not r: continue
-                    if isinstance(r, tuple) and isinstance(r[1], Request):
-                        self.REQUEST_QUEUE.put_nowait(r)
-                    else:
-                        await self.__process_item(priority, r)
-            else:
-                await self.__process_item(priority, result)
+            await self.__process_item(priority, result)
+
+    async def __process_item(self, priority, item):
+        """
+        处理数据管道
+        """
+        for pipe in self.ITEM_PIPELINES:
+            item = pipe(item)
+            if isinstance(item, Coroutine): item = await item
+            if item is None:
+                self._msg_item_dropped += 1
+
+        # 更新 Item 信息
+        self._msg_items += 1
+        self._msg_item_speed = self._msg_items / (self._msg_runtime or 1)
+        if str(priority) not in self._priority_item_map.keys(): self._priority_item_map[str(priority)] = 0
+        self._priority_item_map[str(priority)] += 1
+
+    def __pretty_error(self, err):
+        self.logger.error('-' * 100)
+        f_max = 0
+        funcname = err.__traceback__.tb_frame.f_code.co_name
+        lineno = err.__traceback__.tb_lineno
+        filepath = err.__traceback__.tb_frame.f_code.co_filename
+        if len(funcname) > f_max: f_max = len(funcname)
+        if len(err.__class__.__name__) > f_max: f_max = len(err.__class__.__name__)
+        e_msgs = [(funcname, filepath, lineno)]
+
+        tb = err.__traceback__
+        while tb.tb_next:
+            funcname, lineno, filepath = self.__get_traceback_info(tb.tb_next)
+            if len(funcname) > f_max: f_max = len(funcname)
+
+            e_msgs.append((funcname, filepath, lineno))
+            tb = tb.tb_next
+
+        msg_fmt = '{:>%ss} | File "{}", line {}' % f_max
+        ctt_fmt = '{:>%ss} | {}' % f_max
+        for e_msg in e_msgs:
+
+            content = []
+            index = 1
+            with open('{}'.format(e_msg[1]), 'r') as f:
+                while index <= e_msg[-1] + 1:
+                    line = f.readline()
+                    if index in [e_msg[-1] - 1, e_msg[-1], e_msg[-1] + 1]:
+                        content.append(line.rstrip('\n'))
+                    index += 1
+            self.logger.error(msg_fmt.format(*e_msg))
+            for i, c in enumerate(content):
+                if i == 1:
+                    self.logger.info(ctt_fmt.format('-->', c))
+                else:
+                    self.logger.info(ctt_fmt.format('', c))
+        for a in err.args:
+            self.logger.error(ctt_fmt.format(err.__class__.__name__, a))
+            # self.logger.error('{:>s}: {}'.format('ad', a))
+
+        self.logger.error('')
+
+    @staticmethod
+    def __get_traceback_info(tb):
+        funcname = tb.tb_frame.f_code.co_name
+        lineno = tb.f_lineno if hasattr(tb, 'f_lineno') else tb.tb_frame.f_lineno
+        filepath = tb.tb_frame.f_code.co_filename
+        return funcname, lineno, filepath
 
     async def __downloader(self):
         """
@@ -305,23 +380,28 @@ class Spider(object):
                     for _ in range(self.REQUEST_BATCH_SIZE):
                         param = self.REQUEST_QUEUE.get_nowait()
                         if not param: continue
-                        task = self.__create_task(param)
-                        tasks.append(task)
-
+                        try:
+                            task = self.__create_task(param)
+                        except Exception as e:
+                            self.__pretty_error(e)
+                        else:
+                            tasks.append(task)
                     await asyncio.gather(*tasks)
                     self.SPIDER_STOP_COUNTDOWN = 3
                 else:
                     self.SPIDER_STOP_COUNTDOWN -= 1
-                    if self.SPIDER_STOP_COUNTDOWN < 0:
-                        self.__close_msg()
             else:
                 if self.REQUEST_QUEUE.empty():
                     self.SPIDER_STOP_COUNTDOWN -= 1
                 else:
                     self.SPIDER_STOP_COUNTDOWN = 3
                 while not self.REQUEST_QUEUE.empty():
-                    task = self.__create_task(self.REQUEST_QUEUE.get_nowait())
-                    tasks.append(task)
+                    try:
+                        task = self.__create_task(self.REQUEST_QUEUE.get_nowait())
+                    except Exception as e:
+                        self.__pretty_error(e)
+                    else:
+                        tasks.append(task)
                 await asyncio.gather(*tasks)
 
         if hasattr(self.REQUEST_QUEUE, '_finished'):
@@ -335,27 +415,9 @@ class Spider(object):
             )
         )
 
-    async def __process_item(self, priority, item):
-        """
-        处理数据管道
-        """
-        try:
-            for pipe in self.ITEM_PIPELINES:
-                item = pipe(item)
-                if isinstance(item, Coroutine): item = await item
-                if item is None:
-                    self._msg_item_dropped += 1
-        except Exception as e:
-            self.logger.exception(e)
-        else:
-            # 更新 Item 信息
-            self._msg_items += 1
-            self._msg_item_speed = self._msg_items / (self._msg_runtime or 1)
-            if str(priority) not in self._priority_item_map.keys(): self._priority_item_map[str(priority)] = 0
-            self._priority_item_map[str(priority)] += 1
-
     def __update_msg(self, priority):
         self._msg_runtime = time.time() - self._start_time
+        self._msg_runtime_fmt = human_time(time.time() - self._start_time)
         self._msg_total_requests += 1
         self._msg_download_speed = self._msg_total_requests / (self._msg_runtime or 1)
 
@@ -419,7 +481,7 @@ class Spider(object):
 
     def __collect_msg(self, priority, req):
 
-        return '[{}] [{:.2f}] [{}/{:.2f}] [{}/{:.2f}] {} {}'.format(
+        return '[{}] [{:.2f}] [R {}/{:.2f}] [I {}/{:.2f}] {} {}'.format(
             priority,
             self._msg_runtime,
             self._msg_total_requests,
@@ -439,7 +501,8 @@ class Spider(object):
             'Speed', {k: v for k, v in self.msg.items() if 'speed' in k}
         ))
         self.logger.info('Spider - {:21s}: {}'.format(
-            'Download', {k: v for k, v in self.msg.items() if 'speed' not in k and 'dropped' not in k}
+            'Download', {k: v if k != 'runtime' else human_time(v) for k, v in self.msg.items() if
+                         'speed' not in k and 'dropped' not in k}
         ))
         self.logger.info('Spider - {:21s}: {}'.format(
             'Priority Item',
