@@ -1,17 +1,9 @@
 from espider import Spider
-import time
-
-
-async def print_url(u):
-    print(u.url)
-    return u
 
 
 class Wall(Spider):
     def prepare(self):
         self.REQUEST_BATCH_SIZE = 10
-        self.RESPONSE_MIDDLEWARES = print_url
-        pass
 
     def start_requests(self):
         url = 'https://desk.zol.com.cn/fengjing/weimeiyijing/{}.html'
