@@ -563,11 +563,12 @@ def random_list(stop, start=0, step=1):
 
 
 def human_time(time_stamp):
-    days = time_stamp // 86400
-    hours = (time_stamp % 86400) // 3600
-    minutes = (time_stamp % 3600) // 60
-    seconds = (time_stamp % 60)
-    return days, hours, minutes, seconds
+    _int, _float = int(str(time_stamp).split('.')[0]), int(str(time_stamp).split('.')[1])
+    days = _int // 86400
+    hours = (_int % 86400) // 3600
+    minutes = (_int % 3600) // 60
+    seconds = (_int % 60)
+    return '{} {}:{}:{},{}'.format(days, hours, minutes, seconds, _float)
 
 
 def get_md5(*args):
